/*--------------------------------------------------------------------------+
|                                                                          |
| Copyright 2022 fortiss GmbH						                       |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.ext.variability.ui.properties;

import static org.eclipse.swt.SWT.CHECK;

import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.fortiss.tooling.kernel.service.ICommandStackService;
import org.fortiss.tooling.kernel.ui.extension.base.PropertySectionBase;
import org.fortiss.variability.model.features.AbstractAlternativeFeature;
import org.fortiss.variability.model.features.AbstractFeature;

/**
 * Property Section for {@link AbstractFeature}s.
 * 
 * @author bayha
 */
public class FeaturePropertySectionBase extends PropertySectionBase {

	/** The feature, displayed. */
	protected AbstractFeature feature;

	/** Checkbox to set optionality. */
	protected Button optionalCheckbox;

	/** Label for {@link #optionalCheckbox}. */
	protected Label optionalLabel;

	/** {@inheritDoc} */
	@Override
	protected void setSectionInput(Object input) {
		if(input instanceof AbstractFeature) {
			feature = (AbstractFeature)input;

			if(optionalCheckbox != null) {
				if(feature.eContainer() instanceof AbstractAlternativeFeature) {
					optionalCheckbox.setEnabled(false);
				} else
					optionalCheckbox.setSelection(feature.isOptional());
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public void createControls(Composite parent, TabbedPropertySheetPage tabbedPropertySheetPage) {
		super.createControls(parent, tabbedPropertySheetPage);

		optionalCheckbox = getWidgetFactory().createButton(composite, "", CHECK);
		optionalLabel = createFormEntry(optionalCheckbox, "Optional");
		optionalCheckbox.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				if(feature != null && e.getSource() instanceof Button) {
					ICommandStackService.getInstance().runAsCommand(feature, new Runnable() {
						@Override
						public void run() {
							feature.setOptional(((Button)e.getSource()).getSelection());
						}
					});
				}
			}
		});
	}
}
